@@ -1,18 +1,22 @@
 package Graph;
 
-# COMMENT THESE OUT FOR TESTING AND PRODUCTION.
-# $SIG{__DIE__ } = sub { use Carp; confess };
-# $SIG{__WARN__} = sub { use Carp; confess };
-
 use strict;
 
+BEGIN {
+    if (0) { # SET THIS TO ZERO FOR TESTING AND RELEASES!
+	$SIG{__DIE__ } = \&__carp_confess;
+	$SIG{__WARN__} = \&__carp_confess;
+    }
+    sub __carp_confess { require Carp; Carp::confess(@_) }
+}
+
 use Graph::AdjacencyMap qw(:flags :fields);
 
 use vars qw($VERSION);
 
-$VERSION = '0.58';
+$VERSION = '0.69';
 
-require 5.005;
+require 5.006; # Weak references are required.
 
 use Graph::AdjacencyMap::Heavy;
 use Graph::AdjacencyMap::Light;
@@ -55,19 +59,24 @@ sub _U () { 5 } # Union-Find.
 # - _UNORDUNIQ: just a union of _UNORD and UNIQ
 # Vertices are assumed to be _UNORDUNIQ; edges assume none of these flags.
 
-use Graph::Attribute array => _A, map => "graph";
+use Graph::Attribute array => _A, map => 'graph';
 
 sub _COMPAT02 () { 0x00000001 }
 
 sub stringify {
     my $g = shift;
     my $o = $g->is_undirected;
-    my $e = $o ? "=" : "-";
-    my @e = map { my @v = map { ref $_ ? "[" . join(" ", @$_). "]" : $_ }
-		  @$_;
-		  join($e, $o ? sort @v : @v) } $g->edges05;
-    my @s = sort @e;
-    push @s, sort $g->isolated_vertices;
+    my $e = $o ? '=' : '-';
+    my @e =
+	map {
+	    my @v =
+		map {
+		    ref($_) eq 'ARRAY' ? "[" . join(" ", @$_) . "]" : "$_"
+		}
+	    @$_;
+	    join($e, $o ? sort { "$a" cmp "$b" } @v : @v) } $g->edges05;
+    my @s = sort { "$a" cmp "$b" } @e;
+    push @s, sort { "$a" cmp "$b" } $g->isolated_vertices;
     join(",", @s);
 }
 
@@ -84,8 +93,6 @@ use overload
     'eq' => \&eq,
     'ne' => \&ne;
 
-require Data::Dumper; # for _dump
-
 sub _opt {
     my ($opt, $flags, %flags) = @_;
     while (my ($flag, $FLAG) = each %flags) {
@@ -141,7 +148,7 @@ sub new {
     my $gflags = 0;
     my $vflags;
     my $eflags;
-    my %opt = @_;
+    my %opt = _get_options( \@_ );
 
     if (ref $class && $class->isa('Graph')) {
 	no strict 'refs';
@@ -263,7 +270,7 @@ sub new {
 
     $g->[ _F ] = $gflags;
     $g->[ _G ] = 0;
-    $g->[ _V ] = ($vflags & _HYPER) ?
+    $g->[ _V ] = ($vflags & (_HYPER | _MULTI)) ?
 	Graph::AdjacencyMap::Heavy->_new($uflags, 1) :
 	    (($vflags & ~_UNORD) ?
 	     Graph::AdjacencyMap::Vertex->_new($uflags, 1) :
@@ -362,7 +369,8 @@ sub vertices05 {
     my $g = shift;
     my @v = $g->[ _V ]->paths( @_ );
     if (wantarray) {
-	return $g->[ _V ]->_is_HYPER ? @v : map { @$_ } @v;
+	return $g->[ _V ]->_is_HYPER ?
+	    @v : map { ref $_ eq 'ARRAY' ? @$_ : $_ } @v;
     } else {
 	return scalar @v;
     }
@@ -583,7 +591,10 @@ sub add_vertex_by_id {
 sub add_vertex_get_id {
     my $g = shift;
     $g->expect_multivertexed;
-    return $g->[ _V ]->set_path_by_multi_id( @_, _GEN_ID );
+    my $id = $g->[ _V ]->set_path_by_multi_id( @_, _GEN_ID );
+    $g->[ _G ]++;
+    $g->_union_find_add_vertex( @_ ) if $g->has_union_find;
+    return $id;
 }
 
 sub has_vertex_by_id {
@@ -623,11 +634,12 @@ sub add_edge_by_id {
 
 sub add_edge_get_id {
     my $g = shift;
-    unless ($g->is_multiedged) {
-	require Carp;
-	Carp::croak("Graph::add_edge_get_id: graph not multiedged");
-    }
-    return $g->[ _E ]->set_path( $g->_add_edge( @_ ), _GEN_ID );
+    $g->expect_multiedged;
+    my @i = $g->_add_edge( @_ );
+    my $id = $g->[ _E ]->set_path_by_multi_id( @i, _GEN_ID );
+    $g->_union_find_add_edge( @i ) if $g->has_union_find;
+    $g->[ _G ]++;
+    return $id;
 }
 
 sub has_edge_by_id {
@@ -654,7 +666,9 @@ sub delete_edge_by_id {
 sub get_multiedge_ids {
     my $g = shift;
     $g->expect_multiedged;
-    $g->[ _E ]->get_multi_ids( $g->_vertex_ids( @_ ) );
+    my @id = $g->_vertex_ids( @_ );
+    return unless @id;
+    $g->[ _E ]->get_multi_ids( @id );
 }
 
 ###
@@ -677,7 +691,7 @@ sub vertices_at {
     while (my ($i, $v) = each %{ $Vi }) {
 	my %i;
 	@i{ @i } = @i if @i; # @todo: nonuniq hyper vertices?
-	for my $u (ref $v ? @$v : $v) {
+	for my $u ((ref($v) eq 'ARRAY') ? @$v : $v) {
 	    my $j = exists $v{ $u } ? $v{ $u } : ( $v{ $u } = $i );
 	    if (defined $j && exists $i{ $j }) {
 		delete $i{ $j };
@@ -699,7 +713,9 @@ sub _edges_at {
     my $en = 0;
     my %ev;
     for my $v ( ($V->[_f ] & _HYPER) ? $g->vertices_at( @_ ) : @_ ) {
-	my $vi = $V->_get_path_id( ref $v ? @$v : $v );
+#use Carp;
+#confess("v: $v, ref=".ref($v)) unless ref($v) eq 'ARRAY'; # Added by bsb
+	my $vi = $V->_get_path_id( (ref($v) eq 'ARRAY') ? @$v : $v );
 	next unless defined $vi;
 	my $Ei = $E->_ids;
 	while (my ($ei, $ev) = each %{ $Ei }) {
@@ -727,17 +743,19 @@ sub _edges_from {
     my $en = 0;
     my %ev;
     for my $v ( ($V->[_f ] & _HYPER) ? $g->vertices_at( @_ ) : @_ ) {
-	my $vi = $V->_get_path_id( ref $v ? @$v : $v );
+	my $vi = $V->_get_path_id( (ref($v) eq 'ARRAY') ? @$v : $v );
 	next unless defined $vi;
 	my $Ei = $E->_ids;
 	if (wantarray) {
 	    if ($o) {
 		while (my ($ei, $ev) = each %{ $Ei }) {
+		    next unless @$ev;
 		    push @e, [ $ei, $ev ]
 			if ($ev->[0] == $vi || $ev->[-1] == $vi) && !$ev{$ei}++;
 		}
 	    } else {
 		while (my ($ei, $ev) = each %{ $Ei }) {
+		    next unless @$ev;
 		    push @e, [ $ei, $ev ]
 			if $ev->[0] == $vi && !$ev{$ei}++;
 		}
@@ -745,11 +763,12 @@ sub _edges_from {
 	} else {
 	    if ($o) {
 		while (my ($ei, $ev) = each %{ $Ei }) {
-		    $en++
-			if ($ev->[0] == $vi || $ev->[-1] == $vi);
+		    next unless @$ev;
+		    $en++ if ($ev->[0] == $vi || $ev->[-1] == $vi);
 		}
 	    } else {
 		while (my ($ei, $ev) = each %{ $Ei }) {
+		    next unless @$ev;
 		    $en++ if $ev->[0] == $vi;
 		}
 	    }
@@ -775,17 +794,19 @@ sub _edges_to {
     my $en = 0;
     my %ev;
     for my $v ( ($V->[_f ] & _HYPER) ? $g->vertices_at( @_ ) : @_ ) {
-	my $vi = $V->_get_path_id( ref $v ? @$v : $v );
+	my $vi = $V->_get_path_id( (ref($v) eq 'ARRAY') ? @$v : $v );
 	next unless defined $vi;
 	my $Ei = $E->_ids;
 	if (wantarray) {
 	    if ($o) {
 		while (my ($ei, $ev) = each %{ $Ei }) {
+		    next unless @$ev;
 		    push @e, [ $ei, $ev ]
 			if ($ev->[-1] == $vi || $ev->[0] == $vi) && !$ev{$ei}++;
 		}
 	    } else {
 		while (my ($ei, $ev) = each %{ $Ei }) {
+		    next unless @$ev;
 		    push @e, [ $ei, $ev ]
 			if $ev->[-1] == $vi && !$ev{$ei}++;
 		}
@@ -793,10 +814,12 @@ sub _edges_to {
 	} else {
 	    if ($o) {
 		while (my ($ei, $ev) = each %{ $Ei }) {
+		    next unless @$ev;
 		    $en++ if $ev->[-1] == $vi || $ev->[0] == $vi;
 		}
 	    } else {
 		while (my ($ei, $ev) = each %{ $Ei }) {
+		    next unless @$ev;
 		    $en++ if $ev->[-1] == $vi;
 		}
 	    }
@@ -926,7 +949,7 @@ sub delete_edges {
 sub _in_degree {
     my $g = shift;
     return undef unless @_ && $g->has_vertex( @_ );
-    my $in = 0;
+    my $in =  $g->is_undirected && $g->is_self_loop_vertex( @_ ) ? 1 : 0;
     $in += $g->get_edge_count( @$_ ) for $g->edges_to( @_ );
     return $in;
 }
@@ -939,7 +962,7 @@ sub in_degree {
 sub _out_degree {
     my $g = shift;
     return undef unless @_ && $g->has_vertex( @_ );
-    my $out = 0;
+    my $out =  $g->is_undirected && $g->is_self_loop_vertex( @_ ) ? 1 : 0;
     $out += $g->get_edge_count( @$_ ) for $g->edges_from( @_ );
     return $out;
 }
@@ -1361,7 +1384,7 @@ sub set_edge_attribute_by_id {
     $g->expect_multiedged;
     my $value = pop;
     my $attr  = pop;
-    $g->add_edge_by_id( @_ ) unless $g->has_edge_by_id( @_ );
+    # $g->add_edge_by_id( @_ ) unless $g->has_edge_by_id( @_ );
     my $id = pop;
     $g->[ _E ]->_set_path_attr( $g->_vertex_ids( @_ ), $id, $attr, $value );
 }
@@ -1580,7 +1603,7 @@ sub add_edges {
 
 sub copy {
     my $g = shift;
-    my %opt = @_;
+    my %opt = _get_options( \@_ );
     
     my $c = (ref $g)->new(directed => $g->directed ? 1 : 0,
 			  compat02 => $g->compat02 ? 1 : 0);
@@ -1592,10 +1615,12 @@ sub copy {
 *copy_graph = \&copy;
 
 sub deep_copy {
+    require Data::Dumper;
     my $g = shift;
     my $d = Data::Dumper->new([$g]);
     use vars qw($VAR1);
     $d->Purity(1)->Terse(1)->Deepcopy(1);
+    $d->Deparse(1) if $] >= 5.008;
     eval $d->Dump;
 }
 
@@ -1975,17 +2000,53 @@ sub expect_multiedged {
     _expected('multiedged') unless $g->is_multiedged;
 }
 
+sub _get_options {
+    my @caller = caller(1);
+    unless (@_ == 1 && ref $_[0] eq 'ARRAY') {
+	die "$caller[3]: internal error: should be called with only one array ref argument, at $caller[1] line $caller[2].\n";
+    }
+    my @opt = @{ $_[0] };
+    unless (@opt  % 2 == 0) {
+	die "$caller[3]: expected an options hash, got a non-even number of arguments, at $caller[1] line $caller[2].\n";
+    }
+    return @opt;
+}
+
 ###
 # Random constructors and accessors.
 #
 
+sub fisher_yates_shuffle (@) {
+    # From perlfaq4, but modified to be non-modifying.
+    my @a = @_;
+    my $i = @a;
+    while ($i--) {
+	my $j = int rand ($i+1);
+	@a[$i,$j] = @a[$j,$i];
+    }
+    return @a;
+}
+
+BEGIN {
+    sub _shuffle(@);
+    # Workaround for the Perl bug [perl #32383] where -d:Dprof and
+    # List::Util::shuffle do not like each other: if any debugging
+    # (-d) flags are on, fall back to our own Fisher-Yates shuffle.
+    # The bug was fixed by perl changes #26054 and #26062, which
+    # went to Perl 5.9.3.  If someone tests this with a pre-5.9.3
+    # bleadperl that calls itself 5.9.3 but doesn't yet have the
+    # patches, oh, well.
+    *_shuffle = $^P && $] < 5.009003 ?
+	\&fisher_yates_shuffle : \&List::Util::shuffle;
+}
+
 sub random_graph {
-    my $class = shift;
-    my %opt = @_;
+    my $class = (@_ % 2) == 0 ? 'Graph' : shift;
+    my %opt = _get_options( \@_ );
     my $random_edge;
-    unless (exists $opt{vertices}) {
+    unless (exists $opt{vertices} && defined $opt{vertices}) {
 	require Carp;
-	Carp::croak("Graph::random_graph: argument 'vertices' missing");
+	Carp::croak("Graph::random_graph: argument 'vertices' missing or undef");
     }
     if (exists $opt{random_seed}) {
 	srand($opt{random_seed});
@@ -2032,8 +2093,8 @@ sub random_graph {
     my @V2 = @V;
     # Shuffle the vertex lists so that the pairs at
     # the beginning of the lists are not more likely.
-    @V1 = shuffle @V1;
-    @V2 = shuffle @V2;
+    @V1 = _shuffle @V1;
+    @V2 = _shuffle @V2;
  LOOP:
     while ($E) {
 	for my $v1 (@V1) {
@@ -2126,7 +2187,7 @@ sub MST_Kruskal {
 }
 
 sub _MST_add {
-    my ($g, $HF, $r, $attr, $unseen) = @_;
+    my ($g, $h, $HF, $r, $attr, $unseen) = @_;
     for my $s ( grep { exists $unseen->{ $_ } } $g->successors( $r ) ) {
 	$HF->add( Graph::MSTHeapElem->new( $r, $s, $g->get_edge_attribute( $r, $s, $attr ) ) );
     }
@@ -2138,12 +2199,11 @@ sub _next_random     { shift; (values %{ $_[0] })[ rand keys %{ $_[0] } ] }
 
 sub _root_opt {
     my $g = shift;
-    my %opt;
-    %opt = @_ == 1 ? ( first_root => $_[0] ) : @_;
+    my %opt = @_ == 1 ? ( first_root => $_[0] ) : _get_options( \@_ );
     my %unseen;
     my @unseen = $g->vertices05;
     @unseen{ @unseen } = @unseen;
-    @unseen = shuffle @unseen;
+    @unseen = _shuffle @unseen;
     my $r;
     if (exists $opt{ start }) {
 	$opt{ first_root } = $opt{ start };
@@ -2177,29 +2237,26 @@ sub _heap_walk {
     my ($g, $h, $add, $etc) = splice @_, 0, 4;
 
     my ($opt, $unseenh, $unseena, $r, $next, $code, $attr) = $g->_root_opt(@_);
-    my $HF  = Heap::Fibonacci->new;
+    my $HF = Heap::Fibonacci->new;
 
-    do {
-	if (defined $r) {
-	    $add->($g, $HF, $r, $attr, $unseenh, $etc);
-	    delete $unseenh->{ $r };
-	}
+    while (defined $r) {
+	$add->($g, $h, $HF, $r, $attr, $unseenh, $etc);
+	delete $unseenh->{ $r };
 	while (defined $HF->top) {
 	    my $t = $HF->extract_top;
 	    if (defined $t) {
 		my ($u, $v, $w) = $t->val;
+		# print "extracted top: $u $v $w\n";
 		if (exists $unseenh->{ $v }) {
 		    $h->set_edge_attribute($u, $v, $attr, $w);
 		    delete $unseenh->{ $v };
-		    $add->($g, $HF, $v, $attr, $unseenh, $etc);
+		    $add->($g, $h, $HF, $v, $attr, $unseenh, $etc);
 		}
 	    }
 	}
 	return $h unless defined $next;
-	$r = $next ?
-	    ($code ? $next->( $g, $unseenh ) : $next) :
-		shift @$unseena;
-    } while (keys %$unseenh || defined $HF->top);
+	$r = $code ? $next->( $g, $unseenh ) : shift @$unseena;
+    }
 
     return $h;
 }
@@ -2395,7 +2452,7 @@ sub delete_attributes {
 
 sub topological_sort {
     my $g = shift;
-    my %opt = @_;
+    my %opt = _get_options( \@_ );
     my $eic = $opt{ empty_if_cyclic };
     my $hac;
     if ($eic) {
@@ -2483,13 +2540,16 @@ sub _connected_components {
 	    }
 	} else {
 	    my %r; @r{ $g->unique_vertices } = ();
-	    my $root = sub {
+	    my $froot = sub {
+		wantarray ? ( keys %r ? scalar each %r: ()) : each %r;
+	    };
+	    my $nroot = sub {
 		$cc++;
 		wantarray ? ( keys %r ? scalar each %r: ()) : each %r;
 	    };
 	    my $t = Graph::Traversal::DFS->new($g,
-					       first_root => $root,
-					       next_root => $root,
+					       first_root => $froot,
+					       next_root  => $nroot,
 					       pre => sub {
 						   my ($v, $t) = @_;
 						   $cce{ $v } = $cc;
@@ -2516,7 +2576,6 @@ sub connected_component_by_index {
     my ($g, $i) = @_;
     $g->expect_undirected;
     my ($CCE, $CCI) = $g->_connected_components();
-    $i++ unless $g->has_union_find;
     return defined $CCI->{ $i } ? @{ $CCI->{ $i } } : ( );
 }
 
@@ -2565,7 +2624,9 @@ sub connected_graph {
 	$cg->add_vertices($g->vertices);
     } else {
 	for my $cc ( $g->connected_components() ) {
-	    $cg->add_vertex($super_component->($cc));
+	    my $sc = $super_component->($cc);
+	    $cg->add_vertex($sc);
+	    $cg->set_vertex_attribute($sc, 'subvertices', [ @$cc ]);
 	}
     }
     return $cg;
@@ -2780,6 +2841,7 @@ sub strongly_connected_graph {
     for (my $i = 0; $i <  @c; $i++) {
 	my $c = $c[$i];
 	$s->add_vertex( $s[$i] = $hypervertex->($c) );
+	$s->set_vertex_attribute($s[$i], 'subvertices', [ @$c ]);
 	for my $v (@$c) {
 	    $c{$v} = $i;
 	}
@@ -2860,7 +2922,7 @@ sub biconnectivity {
 	do {
 	    my $w;
 	    do {
-		$w = first { !$U{ $v }{ $_ } } shuffle keys %{ $A{ $v } };
+		$w = first { !$U{ $v }{ $_ } } _shuffle keys %{ $A{ $v } };
 		if (defined $w) {
 		    $U{ $v }{ $w }++;
 		    $U{ $w }{ $v }++;
@@ -2887,7 +2949,7 @@ sub biconnectivity {
 		}
 	    } else {
 		my $e;
-		for my $w (shuffle keys %{ $A{ $r } }) {
+		for my $w (_shuffle keys %{ $A{ $r } }) {
 		    unless ($U{ $r }{ $w }) {
 			$e = $r;
 			last;
@@ -3009,7 +3071,8 @@ sub biconnected_graph {
     my ($bc, $v2bc) = ($g->biconnectivity)[1, 3];
     my $bcg = Graph::Undirected->new;
     for my $c (@$bc) {
-	$bcg->add_vertex($super_component->($c));
+	$bcg->add_vertex(my $s = $super_component->($c));
+	$bcg->set_vertex_attribute($s, 'subvertices', [ @$c ]);
     }
     my %k;
     for my $i (0..$#$bc) {
@@ -3045,8 +3108,8 @@ sub bridges {
 #
 
 sub _SPT_add {
-    my ($g, $HF, $r, $attr, $unseen, $etc) = @_;
-    for my $s ( grep { $unseen->{ $_ } } $g->successors( $r ) ) {
+    my ($g, $h, $HF, $r, $attr, $unseen, $etc) = @_;
+    for my $s ( grep { exists $unseen->{ $_ } } $g->successors( $r ) ) {
 	my $t = $g->get_edge_attribute( $r, $s, $attr ) || 1;
 	if ($t < 0) {
 	    require Carp;
@@ -3054,6 +3117,9 @@ sub _SPT_add {
 	}
 	if (!defined($etc->{ $s }) || $etc->{ $r } + $t < $etc->{ $s }) {
 	    $etc->{ $s } = ($etc->{ $r } || 0) + $t;
+	    # print "$r - $s : setting $s to $etc->{ $s }\n";
+	    $h->set_vertex_attribute( $s, $attr, $etc->{ $s } );
+	    $h->set_vertex_attribute( $s, 'p', $r );
 	    $HF->add( Graph::SPTHeapElem->new($r, $s, $t) );
 	}
     }
@@ -3061,17 +3127,41 @@ sub _SPT_add {
 
 sub SPT_Dijkstra {
     my $g = shift;
-    my %etc;
-    $g->_heap_walk($g->new(), \&_SPT_add, \%etc, @_);
+    my %opt = @_ == 1 ? (first_root => $_[0]) : @_;
+    my $first_root = $opt{ first_root };
+    unless (defined $first_root) {
+	$opt{ first_root} = $first_root = $g->random_vertex();
+    }
+    my $spt_di = $g->get_graph_attribute('_spt_di');
+    unless (defined $spt_di && exists $spt_di->{ $first_root } && $spt_di->{ $first_root }->[ 0 ] == $g->[ _G ]) {
+	my %etc;
+	my $sptg = $g->_heap_walk($g->new(), \&_SPT_add, \%etc, %opt);
+	$spt_di->{ $first_root } = [ $g->[ _G ], $sptg ];
+	$g->set_graph_attribute('_spt_di', $spt_di);
+    }
+    return $spt_di->{ $first_root }->[ 1 ];
 }
 
 *SSSP_Dijkstra = \&SPT_Dijkstra;
 
 *single_source_shortest_paths = \&SPT_Dijkstra;
 
-sub SPT_Bellman_Ford {
-    my $g = shift;
-    my ($opt, $unseenh, $unseena, $r, $next, $code, $attr) = $g->_root_opt(@_);
+sub SP_Dijkstra {
+    my ($g, $u, $v) = @_;
+    my $sptg = $g->SPT_Dijkstra(first_root => $u);
+    my @path = ($v);
+    my $p;
+    while (defined($p = $sptg->get_vertex_attribute($v, 'p'))) {
+	push @path, $p;
+	$v = $p;
+    }
+    @path = () if @path && $path[-1] ne $u;
+    return reverse @path;
+}
+
+sub _SPT_Bellman_Ford {
+    my ($g, $opt, $unseenh, $unseena, $r, $next, $code, $attr) = @_;
+
     my %d;
     $d{ $r } = 0;
     my %p;
@@ -3116,17 +3206,52 @@ sub SPT_Bellman_Ford {
 	}
     }
 
-    my $h = $g->new();
-    for my $v (keys %p) {
-	$h->add_edge( $p{ $v }, $v );
-#	$h->set_vertex_attribute( $v, $attr, $d{ $v } );
+    return (\%p, \%d );
+}
+
+sub SPT_Bellman_Ford {
+    my $g = shift;
+
+    my ($opt, $unseenh, $unseena, $r, $next, $code, $attr) = $g->_root_opt(@_);
+
+    unless (defined $r) {
+	$r = $g->random_vertex();
     }
 
-    return $h;
+    my $spt_bf = $g->get_graph_attribute('_spt_bf');
+    unless (defined $spt_bf && exists $spt_bf->{ $r } && $spt_bf->{ $r }->[ 0 ] == $g->[ _G ]) {
+	my ($p, $d) = $g->_SPT_Bellman_Ford($opt, $unseenh, $unseena, $r, $next, $code, $attr);
+	my $h = $g->new();
+	for my $v (keys %$p) {
+	    my $u = $p->{ $v };
+	    $h->add_edge( $u, $v );
+	    $h->set_edge_attribute( $u, $v, $attr,
+				    $g->get_edge_attribute($u, $v, $attr));
+	    $h->set_vertex_attribute( $v, $attr, $d->{ $v } );
+	    $h->set_vertex_attribute( $v, 'p', $u );
+	}
+	$spt_bf->{ $r } = [ $g->[ _G ], $h ];
+	$g->set_graph_attribute('_spt_bf', $spt_bf);
+    }
+
+    return $spt_bf->{ $r }->[ 1 ];
 }
 
 *SSSP_Bellman_Ford = \&SPT_Bellman_Ford;
 
+sub SP_Bellman_Ford {
+    my ($g, $u, $v) = @_;
+    my $sptg = $g->SPT_Dijkstra(first_root => $u);
+    my @path = ($v);
+    my $p;
+    while (defined($p = $sptg->get_vertex_attribute($v, 'p'))) {
+	push @path, $p;
+	$v = $p;
+    }
+    @path = () if @path && $path[-1] ne $u;
+    return reverse @path;
+}
+
 ###
 # Transitive Closure.
 #
@@ -3153,7 +3278,7 @@ sub transitive_closure_matrix {
     my $g = shift;
     my $tcm = $g->get_graph_attribute('_tcm');
     unless (defined $tcm) {
-	my $apsp = $g->APSP_Floyd_Warshall();
+	my $apsp = $g->APSP_Floyd_Warshall(@_);
 	$tcm = $apsp->get_graph_attribute('_tcm');
 	$g->set_graph_attribute('_tcm', $tcm);
     }
@@ -3374,6 +3499,7 @@ sub could_be_isomorphic {
 #
 
 sub _dump {
+    require Data::Dumper;
     my $d = Data::Dumper->new([$_[0]],[ref $_[0]]);
     defined wantarray ? $d->Dump : print $d->Dump;
 }
